@@ -1,13 +1,14 @@
 package WWW::Bugzilla;
 
-$WWW::Bugzilla::VERSION = '0.8';
+$WWW::Bugzilla::VERSION = '1.5';
 
 use strict;
 use warnings;
 use WWW::Mechanize;
+use Fatal qw(:void open opendir);
 use Carp qw(croak carp);
 
-use constant FIELDS => qw( version component status resolution dup_id assigned_to summary bug_number description os platform severity priority cc url add_cc target_milestone status_whiteboard keywords depends_on blocks additional_comments );
+use constant FIELDS => qw( bugzilla_version bugzilla_version_minor version component status resolution dup_id assigned_to summary bug_number description os platform severity priority cc url add_cc target_milestone status_whiteboard keywords depends_on blocks additional_comments );
  
 my %new_field_map = (   product => 'product',
                         version => 'version',
@@ -22,6 +23,10 @@ my %new_field_map = (   product => 'product',
                         cc => 'cc',
                         url => 'bug_file_loc' );
 
+my %other_field_map = (
+    resolution => 'resolution_knob_5',
+    );
+
 my %update_field_map = (product => 'product',
 #                        bug_number => 'id', 	# this cannot be updated
                         platform => 'rep_platform',
@@ -201,7 +206,7 @@ use Class::MethodMaker
 sub init {
     my $self = shift;
     my %args = @_;
-                        
+   
     croak("'server', 'email', and 'password' are all required arguments.") if ( (not $args{server}) or (not $args{email}) or (not $args{password}) ); 
 
 #    croak("'product' required for new bug.") if ( (not $args{product}) and (not $args{bug_number}) );
@@ -219,10 +224,11 @@ sub init {
 
     if ($self->{bug_number}) {
         $self->_get_update_page();
-    } elsif ($self->{'product'}) {
+    } elsif ($self->{product}) {
         $self->_get_new_page();
     }
 
+    $self->check_error();
     return $self;
 }
 
@@ -230,31 +236,31 @@ sub _get_new_page {
     my $self = shift;
                                                                   
     my $mech = $self->{mech};
-                                                                  
     my $new_page = $self->{protocol}.'://'.$self->{server}.'/enter_bug.cgi?product='.$self->{product};
     $mech->get($new_page);
     $self->check_error();
+    $mech->form_name('Create') if ($self->bugzilla_version == 3);
 
     # bail unless OK or Redirect happens
     croak("Cannot open page $new_page") unless ( ($mech->status == '200') or ($mech->status == '404') );
-
 }
 
 sub _get_update_page {
     my $self = shift;
 
     my $mech = $self->{mech};
-
-    my $update_page = $self->{protocol}.'://'.$self->{server}.'/show_bug.cgi?id='.$self->{bug_number};
-    $mech->get($update_page);
-
-    # bail unless OK or Redirect happens
-    croak("Cannot open page $update_page") unless ( ($mech->status == '200') or ($mech->status == '404') );
-
+    $self->_get_form_by_field('quicksearch');
+    $mech->field('quicksearch', $self->{bug_number});
+    $mech->submit();
+    $self->check_error();
+    
+    $mech->form_name("changeform");
     # set fields to chosen values
     foreach my $field ( keys %update_field_map ) {
         if ($mech->current_form->find_input($update_field_map{$field})) {    
             $self->{$field} = $mech->current_form->value( $update_field_map{$field} );
+        } else {
+#            warn "# Couldn't find $field";
         }
     }
 }
@@ -268,7 +274,7 @@ sub _get_form_by_field {
     my $i = 1;
     foreach my $form ($mech->forms()) {
         if ($form->find_input($field)) {
-            $mech->form($i);
+            $mech->form_number($i);
             return;
         }
         $i++;
@@ -293,6 +299,23 @@ sub _login {
     $mech->field('Bugzilla_login', $email);
     $mech->field('Bugzilla_password', $password);
     $mech->submit_form();
+
+    
+    $mech->get($self->{protocol}.'://'.$server.'/');
+
+    if ($mech->content() =~ /<span>Version (\d+)\.(\d+)(\.\d+)?\+?<\/span>/) {
+        $self->bugzilla_version($1);
+        $self->bugzilla_version_minor($2);
+    } elsif ($mech->content() =~ /<p class="header_addl_info">version (\d+)\./smi) {
+        $self->bugzilla_version($1);
+    } else {
+        croak("Unable to verify bugzilla version.");
+    }
+
+    if ($self->bugzilla_version > 2) {
+        $update_field_map{'status'} = 'bug_status';
+        $other_field_map{'resolution'} = 'resolution';
+    }
 }
 
 =item product() version() component() status() assigned_to() resolution() dup_id() assigned_to() summary() bug_number() description() os() platform() severity() priority() cc() url() add_cc() target_milestone() status_whiteboard() keywords() depends_on() blocks() additional_comments()
@@ -399,7 +422,8 @@ sub mark_as_duplicate {
     
     croak("mark_as_duplicate() may not be called until the bug is committed for the first time") if not $self->{bug_number};
 
-    $self->{status} = 'duplicate';
+    $self->{status} = 'RESOLVED';
+    $self->{resolution} = 'DUPLICATE';
     $self->{dup_id} = $dup_id;    
 }
 
@@ -423,34 +447,41 @@ closed
 =cut
 
 sub change_status {
-    my $self = shift;
-    my $new_status = shift;
+    my ($self, $status) = @_;
 
     croak("change_status() may not be called until the bug is committed for the first time") if not $self->{bug_number};
 
-    $new_status = uc($new_status);
+    $status = uc($status);
 
-    my %status = (  'ASSIGNED'  => 'accept', 
-                    'REOPEN'    => 'reopen',
-                    'VERIFIED'  => 'verify',
-                    'CLOSED'    => 'close' );
+    my %status = (
+            'ASSIGNED'  => 'accept', 
+            'REOPENED'    => 'reopen',
+            'VERIFIED'  => 'verify',
+            'CLOSED'    => 'close'
+            );
 
-    my %resolution = (  'ASSIGNED'  => 1,
-                        'FIXED'     => 1,
-                        'INVALID'   => 1,
-                        'WONTFIX'   => 1,
-                        'LATER'     => 1,
-                        'REMIND'    => 1,
-                        'WORKSFORME' => 1   );
+    my %resolution = (
+            'FIXED'     => 1,
+            'INVALID'   => 1,
+            'WONTFIX'   => 1,
+            'LATER'     => 1,
+            'REMIND'    => 1,
+            'DUPLICATE' => 1,
+            'WORKSFORME' => 1   
+            );
 
-    croak ("$new_status is not a valid status.") if not ($resolution{$new_status} or $status{$new_status});
+    croak ("$status is not a valid status.") if not ($resolution{$status} or $status{$status});
 
-    if ($status{$new_status}) {
-        $self->{status} = $status{$new_status};
+    if ($status{$status}) {
+        $self->{status} = $status;
+        $self->{resolution} = '';
+        # $status{$status};
     } else {
-        $self->{status} = 'resolve';
-        $self->{resolution} = $new_status;
+        $self->{status} = "RESOLVED";
+        $self->{resolution} = $status;
     }
+
+    return 1;
 }
 
 =item add_attachment()
@@ -500,6 +531,8 @@ sub add_attachment {
     my $attach_page = $self->{protocol}.'://'.$self->{server}.'/attachment.cgi?bugid='.$self->{bug_number}.'&action=enter';
     
     $mech->get( $attach_page );
+    $self->check_error();
+    $mech->form_name('entryform');
     $mech->field( 'data', $args{'filepath'} );
     $mech->field( 'description', $args{description} );
     $mech->field( 'comment', $args{comment} ) if $args{comment};
@@ -522,9 +555,11 @@ sub add_attachment {
     $mech->submit_form(); 
     $self->check_error();
     my $id;
-    if ($mech->content =~ /Created/) {
+    if ($mech->content =~ /created/i) {
         my $link = $mech->find_link(text_regex => qr/^Attachment #\d+$/);
-        if ($link->attrs()->{'title'} ne "'" . $args{'description'} . "'" ) {
+        my $title = $link->attrs()->{'title'};
+        
+        if ($title ne "'" . $args{'description'} . "'" && $title ne $args{'description'}) {
             croak('attachment not created');
         }
         if ($link->text =~ /^Attachment #(\d+)$/) {
@@ -555,9 +590,17 @@ sub list_attachments {
     $self->check_error();
     
     my (@attachments);
+    my %seen;
     foreach my $link ($mech->find_all_links(url_regex => qr/attachment\.cgi\?id=\d+$/)) {
-        if ($link->url() =~ /attachment.cgi\?id=(\d+)$/) {
-            push (@attachments, { id => $1, name => $link->text() });
+        if ($link->url() =~ /^attachment.cgi\?id=(\d+)$/) {
+            my $id = $1;
+            next if ($seen{$id});
+            $seen{$id}++;
+            my $i = $link->url();
+            $i =~ s/\?/\\?/g;
+            my $re = '<a(?: name="a\d+")? href="' . $i . '"(?:\s*title="View the content of the attachment">\s*<b>|>)?<span class="(bz_obsolete)">';
+            my $obsolete = ($mech->content() =~ /$re/smi) ? 1 : 0;
+            push (@attachments, { id => $id, name => $link->text(), obsolete => $obsolete });
         } else {
             croak("WWW::Mechanize find_all_links gave us a bogus URL");
         }
@@ -599,6 +642,44 @@ sub get_attachment {
     return $mech->content();
 }
 
+=item obsolete_attachment()
+
+Mark the specified attachment obsolete.  - will not work for new bugs.
+
+=cut
+
+sub obsolete_attachment {
+    my $self = shift;
+    my %args = @_;
+    my $mech = $self->{mech};
+    
+    croak("obsolete_attachment() may not be called until the bug is committed for the first time") if not $self->{bug_number};
+    
+    croak("You must provide either the 'id' or 'name' of the attachment you wish to obsolete") unless ($args{id} || $args{name});
+    
+    my $bug_page = $self->{protocol}.'://'.$self->{server}.'/show_bug.cgi?id='.$self->{bug_number};
+    $mech->get($bug_page);
+    $self->check_error();
+ 
+    my @links;
+    if ($args{'id'}) {
+        @links = $mech->find_all_links( url => 'attachment.cgi?id=' . $args{'id'} );
+    } elsif ($args{'name'}) {
+        @links = $mech->find_all_links( text => $args{'name'} );
+        if (scalar(@links) > 1) {
+            carp('multiple attachments have the same name, returning the first one');
+        }
+    }
+    croak('No such attachment') if (!@links);
+    $links[0]->[0] = $links[0]->[0] . '&action=edit';
+    $links[0]->[5]->{'href'} = $links[0]->[5]->{'href'} . '&action=edit';
+    $mech->get($links[0]);
+    $mech->form_with_fields('id', 'action', 'contenttypemethod');
+    $mech->tick("isobsolete", 1);
+    $mech->submit();
+    return $mech->content();
+}
+
 
 =item commit()
 
@@ -612,23 +693,55 @@ sub commit {
     my $self = shift;
     my %args = @_;
     my $mech = $self->{mech};
+ 
+#    print $mech->uri() . "\n";
+    if ($mech->content() !~ /a href="index\.cgi\?logout=1">/) {
+        croak("must be logged in to commit bugs");
+    }
 
     if ($self->{bug_number}) {
-        foreach my $field ( keys %update_field_map ) {
-            $mech->field( $update_field_map{$field}, $self->{$field} ) if defined($self->{$field});
-            # handle special cases
+        # bugzilla > 3.0
+        if ($self->bugzilla_version() > 2) {
             if ($self->{resolution}) {
-                $mech->field('resolution', $self->{resolution});
+                $mech->field($update_field_map{'status'}, $self->{'status'});
+                $mech->field($other_field_map{'resolution'}, $self->{resolution});
                 $self->{resolution} = undef;
+                $self->{status} = undef;
+            } elsif ($self->{status}) {
+                $mech->field('bug_status', $self->{'status'});
+                $self->{resolution} = undef;
+                $self->{status} = undef;
             }
-            if ($self->{dup_id}) {
-                $mech->field('dup_id', $self->{dup_id});
-                $self->{dup_id} = undef;
+        } else {
+            if ($self->{resolution}) {
+                $mech->field($update_field_map{'status'}, 'resolve');
+                $mech->field($other_field_map{'resolution'}, $self->{resolution});
+                $self->{resolution} = undef;
+                $self->{status} = undef;
+            } elsif ($self->{status}) {
+                $mech->field($update_field_map{'status'}, $self->{status});
+                $self->{status} = undef;
             }
-            if ($self->{assigned_to}) {
-                $mech->field('assigned_to', $self->{assigned_to});
-                $self->{assigned_to} = undef;
+        }
+        
+        if ($self->{dup_id}) {
+            $mech->field('dup_id', $self->{dup_id});
+            $self->{dup_id} = undef;
+        }
+        if ($self->{assigned_to}) {
+            $mech->field('assigned_to', $self->{assigned_to});
+            $self->{assigned_to} = undef;
+        }
+        foreach my $field ( keys %update_field_map ) {
+            # field is missing
+            if (!$mech->current_form->find_input($update_field_map{$field})) {
+#                warn "# $field is missing";
+                next;
             }
+            
+            # field is hidden 
+            next if $mech->current_form->find_input($update_field_map{$field})->type eq 'hidden';
+            $mech->field( $update_field_map{$field}, $self->{$field} ) if defined($self->{$field});
         }
     } else {
         foreach my $field ( keys %new_field_map ) {
@@ -641,10 +754,33 @@ sub commit {
         }
     }
 
-    $mech->submit_form();
-    $self->check_error();
-    $self->{bug_number} = $mech->current_form->value( 'id' ) if not ($self->{bug_number});
+    # delete the comment such that we don't reuse the same comment again accidentally.
+    delete($self->{'comment'});
 
+    $mech->submit_form();
+        
+    # 3.3+ token checking
+    if ($mech->content() =~ /You submitted changes to process_bug\.cgi with an invalid/) {
+        $mech->form_name('check');
+        $mech->submit_form();
+    }
+
+
+    $self->check_error();
+    if (!$self->{bug_number}) {
+        if ($mech->content() =~ /<h2>Bug (\d+) has been added to the database/) {
+            $self->{bug_number} = $1;
+        } elsif ($mech->content() =~ />Bug (\d+)<\/a><\/i> has been added to the database<\/dt>/) {
+            $self->{bug_number} = $1;
+        } elsif ($mech->content() =~ /Bug (\d+) Submitted</) {
+            $self->{bug_number} = $1;
+        } elsif ($mech->content() =~ /Bug&nbsp;(\d+) Submitted</) {
+            $self->{bug_number} = $1;
+        } else {
+#           warn $mech->content();
+            croak("bug was not saved");
+        }
+    }
     $self->_get_update_page() unless ($args{finished});
 
     return $self->{bug_number};
@@ -662,6 +798,10 @@ sub check_error {
     
     if ($mech->content() =~ /<td bgcolor="#ff0000">[\s\r\n]*<font size="\+2">[\s\r\n]*(.*?)[\s\r\n]*<\/font>[\s\r\n]*<\/td>/smi) {
         croak("error : $1");
+    } elsif ($mech->content() =~ /<td id="error_msg" class="throw_error">\s*(.*?)\s*<\/td>/smi) {
+        croak("error : $1");
+    } elsif ($mech->content() =~ /<div class="throw_error">\s*(.*?)<\/div>/smi) {
+        croak("error : $1");
     }
 }
 
@@ -676,6 +816,10 @@ sub get_products {
     my $mech = $self->{mech};
     
     my $url = $self->{protocol}.'://'.$self->{server}.'/enter_bug.cgi';
+    # version >= 3.0
+    if ($self->bugzilla_version == 3) {
+        $url .= '?classification=__all';
+    }
     $mech->get($url);
     $self->check_error();
 
@@ -687,6 +831,7 @@ sub get_products {
     return (@products);
 }
 
+
 =item get_comments()
 
 Lists comments made on an existing bug - will not work for new bugs.
@@ -709,6 +854,14 @@ sub get_comments {
         chomp($comment);
         push (@comments, $comment);
     }
+
+    # 3.3+
+    while ($content =~ m/<pre class="bz_comment_text"  id="comment_text_\d+">\s*(.*?)<\/pre>/smg) {
+        my $comment = $1;
+        chomp($comment);
+        push (@comments, $comment);
+    }
+
     return (@comments);
 }
 
@@ -716,10 +869,10 @@ sub get_comments {
 
 =head1 BUGS, IMPROVEMENTS
 
-There may well be bugs in this module.  Using it as I have, I just
-have not run into any.  In addition, this module does not support 
-ALL of Bugzilla's features.  I will consider any patches or improvements,
-just send me an email at the address listed below.
+There may well be bugs in this module.  Using it as I have, I just have not run
+into any.  In addition, this module does not support ALL of Bugzilla's
+features.  I will consider any patches or improvements, just send me an email
+at the address listed below.
  
 =head1 AUTHOR
 
@@ -727,7 +880,7 @@ Maintained by:
     Brian Caswell, bmc@shmoo.com
 
 Originally written by:
-Matthew C. Vella, the_mcv@yahoo.com
+    Matthew C. Vella, the_mcv@yahoo.com
 
 =head1 LICENSE
                                                                       
